#include <LPC213x.h>

#define DB0				(0x1<<0)
#define DB1				(0x1<<1)
#define DB2				(0x1<<2)
#define DB3				(0x1<<3)
#define DB4				(0x1<<4)
#define DB5				(0x1<<5)
#define DB6				(0x1<<6)
#define DB7				(0x1<<7)
#define RS				(0x1<<16)
#define EN				(0x1<<18)

#define DATA 			DB0+DB1+DB2+DB3+DB4+DB5+DB6+DB7
#define ALL				DATA+RS+EN

#define CLR				IOCLR0 |= 
#define SET				IOSET0 |=

#define FIRST_STR()		CMD_LCD(0x80+0x00)	// 80 -  "   " ; 00 -  ,  
#define SECOND_STR()	CMD_LCD(0x80+0x40)  // 80 -  "   " ; 40 -  ,  
#define CLEAR_LCD()		CMD_LCD(0x01)
#define CURSOR_BLINK()  CMD_LCD(0x0F)
#define CURSOR_ON()		CMD_LCD(0x0E)
#define CURSOR_OFF()	CMD_LCD(0x0C)


void pause(int p);
void MOV(int dig);
void WAIT_LCD(void);
void INIT_LCD(void);
void CMD_LCD(int cmd);
void WRITE_CHAR(unsigned char c);	
void WRITE_TEXT(unsigned char* str);
void WRITE_TEXT_DEL(unsigned char* str);

int main()
{
	IODIR0 = ALL;

	INIT_LCD();
	CLEAR_LCD();
	WRITE_CHAR(' ');
	
	FIRST_STR();
	WRITE_TEXT_DEL("-== LPC2138 ==-");
	SECOND_STR();
	WRITE_TEXT_DEL("LCD PROGRAMMING");

	CURSOR_BLINK();
	while(1);
}

void pause(int p)
{
	int i;
	for(i=0;i<p;i++);
}

void MOV(int dig)
{
	SET dig;
	CLR dig^(0xFF); 
}
void WAIT_LCD()
{
	int i;
	for (i=0; i<2800; i++);
}

void INIT_LCD()
{
	CMD_LCD(0x38);
	CMD_LCD(0x0E);
	CMD_LCD(0x06);
}

void CMD_LCD(int cmd)
{
	CLR RS;
	MOV(cmd);		//  
	SET EN;
	CLR EN;
	WAIT_LCD();	
}

void WRITE_CHAR(unsigned char c)
{
	SET RS;
	MOV(c);			//    
	SET EN;
	CLR EN;
	WAIT_LCD();
}

void WRITE_TEXT(unsigned char* str)
{
	int i;
	for (i=0; i<16 && str[i]!=0; i++)
		WRITE_CHAR(str[i]);
}

void WRITE_TEXT_DEL(unsigned char* str)
{
	int i;

	for (i=0; i<16 && str[i]!=0; i++)
	{
		WRITE_CHAR(str[i]);
		pause(100000);
	}
}
